@@ -9,6 +9,8 @@ BEGIN {
     __PACKAGE__->mk_accessors(qw/store credential name config/);
 };
 
+## Add use_session config item to realm.
+
 sub new {
     my ($class, $realmname, $config, $app) = @_;
 
@@ -17,6 +19,14 @@ sub new {
     
     $self->name($realmname);
     
+    if (!exists($self->config->{'use_session'})) {
+        if (exists($app->config->{'Plugin::Authentication'}{'use_session'})) {
+            $self->config->{'use_session'} = $app->config->{'Plugin::Authentication'}{'use_session'};
+        } else {
+            $self->config->{'use_session'} = 1;
+        }
+    }
+
     $app->log->debug("Setting up auth realm $realmname") if $app->debug;
 
     # use the Null store as a default
@@ -54,14 +64,22 @@ sub new {
     ###  we must eval the ensure_class_loaded - because we might need to try the old-style
     ###  ::Plugin:: module naming if the standard method fails. 
     
+    ## Note to self - catch second exception and bitch in detail?
+    
     eval {
         Catalyst::Utils::ensure_class_loaded( $credentialclass );
     };
     
     if ($@) {
         $app->log->warn( qq(Credential class "$credentialclass" not found, trying deprecated ::Plugin:: style naming. ) );
+        my $origcredentialclass = $credentialclass;
         $credentialclass =~ s/Catalyst::Authentication/Catalyst::Plugin::Authentication/;
-        Catalyst::Utils::ensure_class_loaded( $credentialclass );
+
+        eval { Catalyst::Utils::ensure_class_loaded( $credentialclass ); };
+        if ($@) {
+            Carp::croak "Unable to load credential class, " . $origcredentialclass . " OR " . $credentialclass . 
+                        " in realm " . $self->name;
+        }
     }
     
     eval {
@@ -70,8 +88,13 @@ sub new {
     
     if ($@) {
         $app->log->warn( qq(Store class "$storeclass" not found, trying deprecated ::Plugin:: style naming. ) );
+        my $origstoreclass = $storeclass;
         $storeclass =~ s/Catalyst::Authentication/Catalyst::Plugin::Authentication/;
-        Catalyst::Utils::ensure_class_loaded( $storeclass );
+        eval { Catalyst::Utils::ensure_class_loaded( $storeclass ); };
+        if ($@) {
+            Carp::croak "Unable to load store class, " . $origstoreclass . " OR " . $storeclass . 
+                        " in realm " . $self->name;
+        }
     }
     
     # BACKWARDS COMPATIBILITY - if the store class does not define find_user, we define it in terms 
@@ -132,19 +155,77 @@ sub authenticate {
      }
 }
 
+sub user_is_restorable {
+    my ($self, $c) = @_;
+    
+    return unless
+         $c->isa("Catalyst::Plugin::Session")
+         and $self->config->{'use_session'}
+         and $c->session_is_valid;
+
+    return $c->session->{__user};
+}
+
+sub restore_user {
+    my ($self, $c, $frozen_user) = @_;
+    
+    $frozen_user ||= $self->user_is_restorable($c);
+    return unless defined($frozen_user);
+
+    my $user = $self->from_session( $c, $frozen_user );
+    
+    if ($user) {
+        $c->_user( $user );
+    
+        # this sets the realm the user originated in.
+        $user->auth_realm($self->name);
+    } else {
+		Catalyst::Exception->throw("Store claimed to have a restorable user, but restoration failed.  Did you change the user's id_field?");
+	}
+	 
+    return $user;
+}
+
+sub persist_user {
+    my ($self, $c, $user) = @_;
+    
+    if (
+        $c->isa("Catalyst::Plugin::Session")
+        and $self->config->{'use_session'}
+        and $user->supports("session") 
+    ) {
+        $c->session->{__user_realm} = $self->name;
+    
+        # we want to ask the store for a user prepared for the session.
+        # but older modules split this functionality between the user and the
+        # store.  We try the store first.  If not, we use the old method.
+        if ($self->store->can('for_session')) {
+            $c->session->{__user} = $self->store->for_session($c, $user);
+        } else {
+            $c->session->{__user} = $user->for_session;
+        }
+    }
+    return $user;
+}
+
+sub remove_persisted_user {
+    my ($self, $c) = @_;
+    
+    if (
+        $c->isa("Catalyst::Plugin::Session")
+        and $self->config->{'use_session'}
+        and $c->session_is_valid
+    ) {
+        delete @{ $c->session }{qw/__user __user_realm/};
+    }    
+}
+
+## backwards compatibility - I don't think many people wrote realms since they
+## have only existed for a short time - but just in case.
 sub save_user_in_session {
     my ( $self, $c, $user ) = @_;
 
-    $c->session->{__user_realm} = $self->name;
-    
-    # we want to ask the store for a user prepared for the session.
-    # but older modules split this functionality between the user and the
-    # store.  We try the store first.  If not, we use the old method.
-    if ($self->store->can('for_session')) {
-        $c->session->{__user} = $self->store->for_session($c, $user);
-    } else {
-        $c->session->{__user} = $user->for_session;
-    }
+    return $self->persist_user($c, $user);
 }
 
 sub from_session {
@@ -188,6 +269,11 @@ user doesn't exist (most useful for remote authentication schemes).
 Set this to true if you wish this realm to auto-update user accounts after
 authentication (most useful for remote authentication schemes).
 
+=item use_session
+
+Sets session usage for this particular realm - overriding the global use_sesion setting.
+
+
 =back
 
 =head1 METHODS
@@ -217,17 +303,48 @@ Performs the authentication process for the current realm.  The default
 realm class simply delegates this to the credential and sets 
 the authenticated user on success.  Returns the authenticated user object;
 
-=head save_user_in_session($c, $user)
+=head1 USER PERSISTENCE
+
+The Realm class allows complete control over the persistance of users
+between requests.  By default the realm attempts to use the Catalyst
+session system to accomplish this.  By overriding the methods below
+in a custom Realm class, however, you can handle user persistance in
+any way you see fit.  
+
+=head2 persist_user($c, $user)
+
+persist_user is the entry point for saving user information between requests
+in most cases this will utilize the session.  By default this uses the 
+catalyst session system to store the user by calling for_session on the
+active store.  The user object must be a subclass of 
+Catalyst::Authentication::User.  If you have updated the user object, you 
+must call persist_user again to ensure that the persisted user object reflects
+your updates.
+
+=head2 remove_persisted_user($c)
+
+Removes any persisted user data.  By default, removes the user from the session.
+
+=head2 user_is_restorable( $c )
+
+Returns whether there is a persisted user that may be restored.  Returns
+a token used to restore the user.  With the default session persistance
+it returns the raw frozen user information.
+
+=head2 restore_user($c, [$frozen_user])
+
+Restores the user from the given frozen_user parameter, or if not provided,
+using the response from $self->user_is_restorable();  Uses $self->from_session()
+to decode the frozen user.
 
-Used to save the user in a session. Saves $user in the current session, 
-marked as originating in the current realm.  Calls $store->for_session() by 
-default.  If for_session is not available in the store class, will attempt
-to call $user->for_session().
 
 =head2 from_session($c, $frozenuser )
 
-Triggers restoring of the user from data in the session. The default realm
-class simply delegates the call to $store->from_session($c, $frozenuser);
+Decodes the frozenuser information provided and returns an instantiated 
+user object.  By default, this call is delegated to $store->from_session().
+
+=head2 save_user_in_session($c, $user)
+
+DEPRECATED.  Use persist_user instead.  (this simply calls persist_user)
 
 =cut
-
